# knapsack.py

from pyomo.environ import *
from pyomo.opt import SolverStatus, TerminationCondition
import pyomo.opt

def create_model(n, c, w, K):
    model = ConcreteModel()
    model.N = Set(initialize=range(n))
    model.x = Var(model.N, within = Binary)
    def objective_function(model):
        return -sum(w[j]*model.x[j] for j in model.N)
    model.obj = Objective(rule = objective_function)
    def knapsack_constraint(model):
        return sum(c[j]*model.x[j] for j in model.N) <= K
    model.knapsack = Constraint(rule=knapsack_constraint)
    return model

# main
n = 3
c = [1,2,3]
w = [3,4,5]
K = 3
instance = create_model(n,c,w,K)
solver = pyomo.opt.SolverFactory('cplex', solver_io = 'nl')
if solver is None:
    raise RuntimeError('solver not found')
solver.options['timelimit'] = 60
results = solver.solve(instance, keepfiles = False, tee = True)
if ((results.solver.status == SolverStatus.ok) and (results.solver.termination_condition == TerminationCondition.optimal)):
    # feasible and optimal
    instance.solutions.load_from(results)
    xstar = [instance.x[i].value for i in instance.N]
    print('solved to optimality')
    print(xstar)
else:
    print('not solved to optimality')
    
