# subsetsum.py

from pyomo.environ import *
from pyomo.opt import SolverStatus, TerminationCondition
import pyomo.opt

def create_model(n, a, b):
    model = ConcreteModel()
    model.N = Set(initialize=range(n))
    model.x = Var(model.N, within = Binary)
    def objective_function(model):
        return 0
    model.obj = Objective(rule = objective_function)
    def subsetsum_constraint(model):
        return sum(a[j]*model.x[j] for j in model.N) == b
    model.subsetsum = Constraint(rule=subsetsum_constraint)
    return model

# main
n = 5
a = [1, 1, 1, 1, 4, 5]
b = 3
instance = create_model(n, a, b)
solver = pyomo.opt.SolverFactory('cplex', solver_io = 'nl')
if solver is None:
    raise RuntimeError('solver not found')
solver.options['timelimit'] = 60
results = solver.solve(instance, keepfiles = False, tee = True)
if ((results.solver.status == SolverStatus.ok) and (results.solver.termination_condition == TerminationCondition.optimal)):
    # feasible and optimal
    instance.solutions.load_from(results)
    xstar = [instance.x[i].value for i in instance.N]
    print('solved to optimality')
    print(xstar)
else:
    print('not solved to optimality')
    
